----------------------------------------------------------------------------------
-- Module Name:    reclock - Behavioral 
-- Author:	Mike Field (hamster@snap.net.nz)
-- Revision:	1.0
--
-- Regenerate the S/PDIF clock to sample the input signal.
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity reclock is
    Port ( clk 					: in  STD_LOGIC;
           onePulse 				: in  STD_LOGIC_VECTOR (7 downto 0);
           flipped  				: in  STD_LOGIC;
           takeSample 			: out  STD_LOGIC;
           resetInputCounter 	: out  STD_LOGIC);
end reclock;

architecture Behavioral of reclock is
	signal halfPulse 				: STD_LOGIC_VECTOR(7 downto 0);
	signal twoPulse 				: STD_LOGIC_VECTOR(8 downto 0);
	signal fourPulse				: STD_LOGIC_VECTOR(9 downto 0);

	signal oneAndAHalfPulse 	: STD_LOGIC_VECTOR(8 downto 0);
	signal twoAndAHalfPulse 	: STD_LOGIC_VECTOR(9 downto 0);
	
	type reclock_reg is record
		count					: STD_LOGIC_VECTOR(9 downto 0);
		takeSample			: STD_LOGIC;
		resetInputCounter : STD_LOGIC;
	end record;
	signal r : reclock_reg := ("0000000000",'0','0');
	signal n : reclock_reg;
begin
	-- Output Assignments
	takeSample 			<= r.takeSample;
	resetInputCounter <= r.resetInputCounter;
	
	-- Simple logic
	halfPulse				<= "0" & onePulse(7 downto 1);
	twoPulse					<= onePulse & "0";
	fourPulse 				<= onePulse & "00";
	oneAndAHalfPulse 		<= ("0" & onePulse) + ("0"  & halfPulse);
	twoAndAHalfPulse 		<= ("0" & twoPulse) + ("00" & halfPulse);

	-- Determine next state 
	process(flipped, r, oneAndAHalfPulse, twoAndAHalfPulse, fourPulse)
	begin
		n.count 					<= r.count+1;
		n.takeSample			<= '0';
		n.resetInputCounter 	<= '0';
		
		if n.count >= fourPulse then
			n.resetInputCounter <= '1';
		end if;

		if n.count  = halfPulse then
			n.takeSample <= '1';
		elsif n.count  = twoAndAHalfPulse then
			n.takeSample <= '1';
		elsif n.count = oneAndAHalfPulse then
			n.takeSample <= '1';
		end if;

		if flipped = '1' then
			n.count <= "0000000001";
		end if;
	end process;
	
	-- Assign next State
	process (clk, n)
	begin
		if clk'event and clk = '1' then
			r <= n;
		end if;
	end process;
end Behavioral;

