----------------------------------------------------------------------------------
-- Module Name: spdifCapture - Behavioral 
-- Author:	Mike Field (hamster@snap.net.nz)
-- Revision:	1.0
--
-- Capture a S/PDIF signal and play it back as audio.
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity spdifCapture is
    Port ( clk : in  STD_LOGIC;
			  pulse	: in  STD_LOGIC;
			  dac_left_out  : out STD_LOGIC;
			  dac_right_out : out STD_LOGIC);
end spdifCapture;

architecture Behavioral of spdifCapture is

	COMPONENT resync
	PORT(
		clk 		: IN std_logic;
		pulse 	: IN std_logic;          
		flipped 	: OUT std_logic;
		synced	: out	STD_LOGIC
		);
	END COMPONENT;

	COMPONENT InputCounter
	PORT(
		Clk 		: IN std_logic;
		Flipped 	: IN std_logic;
		Reset 	: IN std_logic;          
		MinPulse : OUT std_logic_vector(7 downto 0);
		Overflow : OUT std_logic
	);
	END COMPONENT;

	COMPONENT reclock
	PORT(
		clk : IN std_logic;
		onePulse : IN std_logic_vector(7 downto 0);
		flipped : IN std_logic;          
		takeSample : OUT std_logic;
		resetInputCounter : OUT std_logic
		);
	END COMPONENT;

	COMPONENT frameCapture
	PORT(
		clk 			: IN std_logic;
		bitstream 	: IN std_logic;
		takeSample 	: IN std_logic;
		channelA 	: out std_logic;          
		data 			: OUT std_logic_vector(23 downto 0);
		dataValid 	: out std_logic
		);
	END COMPONENT;

	COMPONENT dac16
	PORT(
		clk : IN std_logic;
		data : IN std_logic_vector(15 downto 0);          
		dac_out : OUT std_logic
		);
	END COMPONENT;


	signal pulseLen 		: std_logic_vector(7 downto 0);
	signal minPulse 		: std_logic_vector(7 downto 0);
	signal InputReset 	: std_logic;
	signal Overflow		: std_logic;
	signal invalidPulse 	: std_logic;
	signal flipped			: std_logic;
	signal syncedData 	: std_logic;

	signal takeSample		: std_logic;
	signal sample 			: std_logic_vector(23 downto 0);
	signal left 			: std_logic_vector(23 downto 0);
	signal right 			: std_logic_vector(23 downto 0);
	signal newSubFrame	: std_logic;
	signal channelA		: std_logic;
	signal channel			: std_logic_vector(2 downto 0) := "000";

begin	
	Inst_resync: resync PORT MAP(
		clk => clk,
		pulse => pulse,
		flipped => flipped,
		synced  => syncedData
	);

	Inst_InputCounter: InputCounter PORT MAP(
		-- Inputs
		clk => clk,
		Flipped => Flipped,
		Reset => InputReset,
		-- Outputs
		MinPulse => minPulse,
		Overflow => overflow
	);

	Inst_reclock: reclock PORT MAP(
		clk => clk,
		onePulse => minPulse,
		flipped => flipped,
		takeSample => takeSample,
		resetInputCounter => InputReset
	);

	Inst_frameCapture: frameCapture PORT MAP(
		clk 			=> clk,
		bitstream 	=> syncedData,
		takeSample 	=> takeSample,
		data 			=> sample,
		channelA 	=> channelA,
		dataValid 	=> newSubFrame
	);


	left_dac16: dac16 PORT MAP(
		clk 		=> clk,
		data 		=> left(23 downto 8),
		dac_out 	=> dac_left_out
	);
	
	right_dac16: dac16 PORT MAP(
		clk 		=> clk,
		data 		=> right(23 downto 8),
		dac_out 	=> dac_right_out
	);
	
	process (clk, newSubFrame, channelA, sample)
	begin
		if clk'event and clk = '1' and newSubFrame = '1' then
			if channelA = '1' then
				left <= sample(23 downto 0);
				channel <= "001";
			elsif channel <= "001" then
				right <= sample(23 downto 0);
				channel <= "010";
			end if;
		end if;
	end process;
end Behavioral;
